searchData={"items":[{"type":"behaviour","title":"Spark.Dsl","doc":"The primary entry point for adding a DSL to a module.\n\nTo add a DSL to a module, add `use Spark.Dsl, ...options`. The options supported with `use Spark.Dsl` are:\n\n* `:single_extension_kinds` (list of `t:atom/0`) - The extension kinds that are allowed to have a single value. For example: `[:data_layer]` The default value is `[]`.\n\n* `:many_extension_kinds` (list of `t:atom/0`) - The extension kinds that can have multiple values. e.g `[notifiers: [Notifier1, Notifier2]]` The default value is `[]`.\n\n* `:untyped_extensions?` (`t:boolean/0`) - Whether or not to support an `extensions` key which contains untyped extensions The default value is `true`.\n\n* `:default_extensions` (`t:keyword/0`) - The extensions that are included by default. e.g `[data_layer: Default, notifiers: [Notifier1]]`\n  Default values for single extension kinds are overwritten if specified by the implementor, while many extension\n  kinds are appended to if specified by the implementor. The default value is `[]`.\n\n* `:opt_schema` (`t:keyword/0`) - A schema for additional options to accept when calling `use YourSpark` The default value is `[]`.\n\n\n\nSee the callbacks defined in this module to augment the behavior/compilation of the module getting a Dsl.","ref":"Spark.Dsl.html"},{"type":"behaviour","title":"Schemas/Data Types - Spark.Dsl","doc":"Spark DSLs use a superset of `NimbleOptions` for the `schema` that makes up sections/entities of the DSL.\nFor more information, see `Spark.OptionsHelpers`.","ref":"Spark.Dsl.html#module-schemas-data-types"},{"type":"callback","title":"Spark.Dsl.explain/2","doc":"Validate/add options. Those options will be passed to `handle_opts` and `handle_before_compile`","ref":"Spark.Dsl.html#c:explain/2"},{"type":"callback","title":"Spark.Dsl.handle_before_compile/1","doc":"Handle options in the context of the module, after all extensions have been processed. Must return a `quote` block.","ref":"Spark.Dsl.html#c:handle_before_compile/1"},{"type":"function","title":"Spark.Dsl.handle_fragments/2","doc":"","ref":"Spark.Dsl.html#handle_fragments/2"},{"type":"callback","title":"Spark.Dsl.handle_opts/1","doc":"Handle options in the context of the module. Must return a `quote` block.\n\nIf you want to persist anything in the DSL persistence layer,\nuse `@persist {:key, value}`. It can be called multiple times to\npersist multiple times.","ref":"Spark.Dsl.html#c:handle_opts/1"},{"type":"callback","title":"Spark.Dsl.init/1","doc":"Validate/add options. Those options will be passed to `handle_opts` and `handle_before_compile`","ref":"Spark.Dsl.html#c:init/1"},{"type":"function","title":"Spark.Dsl.is?/2","doc":"","ref":"Spark.Dsl.html#is?/2"},{"type":"callback","title":"Spark.Dsl.verify/2","doc":"A callback that is called in the `after_verify` hook. Only runs on versions of Elixir >= 1.14.0","ref":"Spark.Dsl.html#c:verify/2"},{"type":"type","title":"Spark.Dsl.entity/0","doc":"","ref":"Spark.Dsl.html#t:entity/0"},{"type":"type","title":"Spark.Dsl.opts/0","doc":"","ref":"Spark.Dsl.html#t:opts/0"},{"type":"type","title":"Spark.Dsl.section/0","doc":"","ref":"Spark.Dsl.html#t:section/0"},{"type":"type","title":"Spark.Dsl.t/0","doc":"","ref":"Spark.Dsl.html#t:t/0"},{"type":"module","title":"Spark.Dsl.Builder","doc":"Utilities for building DSL objects programatically, generally used in transformers.","ref":"Spark.Dsl.Builder.html"},{"type":"macro","title":"Spark.Dsl.Builder.defbuilder/2","doc":"","ref":"Spark.Dsl.Builder.html#defbuilder/2"},{"type":"macro","title":"Spark.Dsl.Builder.defbuilderp/2","doc":"","ref":"Spark.Dsl.Builder.html#defbuilderp/2"},{"type":"function","title":"Spark.Dsl.Builder.handle_nested_builders/2","doc":"Handles nested values that may be `{:ok, result}` or `{:error, term}`, returning any errors and unwrapping any ok values\n\nThis allows users of builders to do things like:\n\n```elixir\ndsl_state\n|> Ash.Resource.Builder.add_new_action(:update, :publish,\n  changes: [\n    Ash.Resource.Builder.build_action_change(\n      Ash.Resource.Change.Builtins.set_attribute(:state, :published)\n    )\n  ]\n)\n```\n\nIf your builder function calls `handle_nested_builders/2` with their input before building the thing its building.","ref":"Spark.Dsl.Builder.html#handle_nested_builders/2"},{"type":"type","title":"Spark.Dsl.Builder.input/0","doc":"","ref":"Spark.Dsl.Builder.html#t:input/0"},{"type":"type","title":"Spark.Dsl.Builder.result/0","doc":"","ref":"Spark.Dsl.Builder.html#t:result/0"},{"type":"module","title":"Spark.Dsl.Entity","doc":"Declares a DSL entity.\n\nA dsl entity represents a dsl constructor who's resulting value is a struct.\nThis lets the user create complex objects with arbitrary(mostly) validation rules.\n\nThe lifecycle of creating entities is complex, happening as Elixir is compiling\nthe modules in question. Some of the patterns around validating/transforming entities\nhave not yet solidified. If you aren't careful and don't follow the guidelines listed\nhere, you can have subtle and strange bugs during compilation. Anything not isolated to\nsimple value validations should be done in `transformers`. See `Spark.Dsl.Transformer`.\n\nAn entity has a `target` indicating which struct will ultimately be built. An entity\nalso has a `schema`. This schema is used for documentation, and the options are validated\nagainst it before continuing on with the DSL.\n\nTo create positional arguments to the builder, use `args`. The values provided to\n`args` need to be in the provided schema as well. They will be positional arguments\nin the same order that they are provided in the `args` key.\n\n`auto_set_fields` will set the provided values into the produced struct (they do not need\nto be included in the schema).\n\n`transform` is a function that takes a created struct and can alter it. This happens immediately\nafter handling the DSL options, and can be useful for setting field values on a struct based on\nother values in that struct. If you need things that aren't contained in that struct, use an\n`Spark.Dsl.Transformer`. This function returns `{:ok, new_entity}` or `{:error, error}`, so this can\nalso be used to validate the entity.\n\n`entities` allows you to specify a keyword list of nested entities. Nested entities are stored\non the struct in the corresponding key, and are used in the same way entities are otherwise.\n\n`singleton_entity_keys` specifies a set of entity keys (specified above) that should only have a\nsingle value. This will be validated and unwrapped into `nil` | `single_value` on success.\n\n`identifier` expresses that a given entity is unique by that field, validated by the DSL.","ref":"Spark.Dsl.Entity.html"},{"type":"module","title":"Example - Spark.Dsl.Entity","doc":"```elixir\n@my_entity %Spark.Dsl.Entity{\n  name: :my_entity,\n  target: MyStruct,\n  schema: [my_field: [type: :atom, required: false]]\n}\n```\n\nOnce compiled by Spark, entities can be invoked with a keyword list:\n\n```elixir\nmy_entity my_field: :value\n```\n\nOr with a do block:\n\n```elixir\nmy_entity do\n  my_field :value\nend\n```\n\nFor a full example, see `Spark.Dsl.Extension`.","ref":"Spark.Dsl.Entity.html#module-example"},{"type":"type","title":"Spark.Dsl.Entity.args/0","doc":"Specifies positional arguments for an Entity.\n\nAn entity declared like this:\n\n```elixir\n@entity %Spark.Dsl.Entity{\n  name: :entity,\n  target: Entity,\n  schema: [\n    positional: [type: :atom, required: true],\n    other: [type: :atom, required: false],\n  ],\n  args: [:positional]\n}\n```\n\nCan be instantiated like this:\n\n```elixir\nentity :positional_argument do\n  other :other_argument\nend\n```","ref":"Spark.Dsl.Entity.html#t:args/0"},{"type":"type","title":"Spark.Dsl.Entity.auto_set_fields/0","doc":"Set the provided key value pairs in the produced struct. These fields do not need to be included in the Entity's schema.","ref":"Spark.Dsl.Entity.html#t:auto_set_fields/0"},{"type":"type","title":"Spark.Dsl.Entity.deprecations/0","doc":"","ref":"Spark.Dsl.Entity.html#t:deprecations/0"},{"type":"type","title":"Spark.Dsl.Entity.describe/0","doc":"User provided documentation.\n\nDocumentation provided in a `Entity`'s `describe` field will be included by `Spark` in any generated documentation that includes the `Entity`.","ref":"Spark.Dsl.Entity.html#t:describe/0"},{"type":"type","title":"Spark.Dsl.Entity.docs/0","doc":"Internal field. Not set by user.","ref":"Spark.Dsl.Entity.html#t:docs/0"},{"type":"type","title":"Spark.Dsl.Entity.entities/0","doc":"A keyword list of nested entities.","ref":"Spark.Dsl.Entity.html#t:entities/0"},{"type":"type","title":"Spark.Dsl.Entity.examples/0","doc":"","ref":"Spark.Dsl.Entity.html#t:examples/0"},{"type":"type","title":"Spark.Dsl.Entity.hide/0","doc":"","ref":"Spark.Dsl.Entity.html#t:hide/0"},{"type":"type","title":"Spark.Dsl.Entity.id/0","doc":"","ref":"Spark.Dsl.Entity.html#t:id/0"},{"type":"type","title":"Spark.Dsl.Entity.imports/0","doc":"","ref":"Spark.Dsl.Entity.html#t:imports/0"},{"type":"type","title":"Spark.Dsl.Entity.links/0","doc":"","ref":"Spark.Dsl.Entity.html#t:links/0"},{"type":"type","title":"Spark.Dsl.Entity.modules/0","doc":"","ref":"Spark.Dsl.Entity.html#t:modules/0"},{"type":"type","title":"Spark.Dsl.Entity.name/0","doc":"","ref":"Spark.Dsl.Entity.html#t:name/0"},{"type":"type","title":"Spark.Dsl.Entity.no_depend_modules/0","doc":"","ref":"Spark.Dsl.Entity.html#t:no_depend_modules/0"},{"type":"type","title":"Spark.Dsl.Entity.recursive_as/0","doc":"","ref":"Spark.Dsl.Entity.html#t:recursive_as/0"},{"type":"type","title":"Spark.Dsl.Entity.singleton_entity_keys/0","doc":"","ref":"Spark.Dsl.Entity.html#t:singleton_entity_keys/0"},{"type":"type","title":"Spark.Dsl.Entity.snippet/0","doc":"","ref":"Spark.Dsl.Entity.html#t:snippet/0"},{"type":"type","title":"Spark.Dsl.Entity.t/0","doc":"","ref":"Spark.Dsl.Entity.html#t:t/0"},{"type":"type","title":"Spark.Dsl.Entity.target/0","doc":"Defines the struct that will be built from this entity definition.\n\nThe struct will need to have fields for all [`entities`](#t:entities/0), `t:schema/0` fields, and `t:auto_set_fields/0`.","ref":"Spark.Dsl.Entity.html#t:target/0"},{"type":"type","title":"Spark.Dsl.Entity.transform/0","doc":"Specifies a function that will run on the target struct after building.\n\n```elixir\n@my_entity %Spark.Dsl.Entity{\n  name: :my_entity,\n  target: MyEntity,\n  schema: [\n    my_field: [type: :list, required: true]\n  ],\n  transform: {MyModule, :max_three_items, []}\n}\n\ndef max_three_items(my_entity) do\n  if length(my_entity.my_field) > 3 do\n    {:error, \"Can't have more than three items\"}\n  else\n    {:ok, my_entity}\n  end\nend\n```","ref":"Spark.Dsl.Entity.html#t:transform/0"},{"type":"behaviour","title":"Spark.Dsl.Extension","doc":"An extension to the Spark DSL.\n\nThis allows configuring custom DSL components, whose configurations\ncan then be read back. This guide is still a work in progress, but should\nserve as a decent example of what is possible. Open issues on Github if you\nhave any issues/something is unclear.\n\nThe example at the bottom shows how you might build a (not very contextually\nrelevant) DSL extension that would be used like so:\n\n    defmodule MyApp.Vehicle do\n      use Spark.Dsl\n    end\n\n    defmodule MyApp.MyResource do\n      use MyApp.Vehicle,\n        extensions: [MyApp.CarExtension]\n\n      cars do\n        car :ford, :focus, trim: :sedan\n        car :toyota, :corolla\n      end\n    end\n\nThe extension:\n\n    defmodule MyApp.CarExtension do\n      @car_schema [\n        make: [\n          type: :atom,\n          required: true,\n          doc: \"The make of the car\"\n        ],\n        model: [\n          type: :atom,\n          required: true,\n          doc: \"The model of the car\"\n        ],\n        type: [\n          type: :atom,\n          required: true,\n          doc: \"The type of the car\",\n          default: :sedan\n        ]\n      ]\n\n      @car %Spark.Dsl.Entity{\n        name: :car,\n        describe: \"Adds a car\",\n        examples: [\n          \"car :ford, :focus\"\n        ],\n        target: MyApp.Car,\n        args: [:make, :model],\n        schema: @car_schema\n      }\n\n      @cars %Spark.Dsl.Section{\n        name: :cars, # The DSL constructor will be `cars`\n        describe: \"\"\"\n        Configure what cars are available.\n\n        More, deeper explanation. Always have a short one liner explanation,\n        an empty line, and then a longer explanation.\n        \"\"\",\n        entities: [\n          @car # See `Spark.Dsl.Entity` docs\n        ],\n        schema: [\n          default_manufacturer: [\n            type: :atom,\n            doc: \"The default manufacturer\"\n          ]\n        ]\n      }\n\n      use Spark.Dsl.Extension, sections: [@cars]\n    end\n\n\nOften, we will need to do complex validation/validate based on the configuration\nof other resources. Due to the nature of building compile time DSLs, there are\nmany restrictions around that process. To support these complex use cases, extensions\ncan include `transformers` which can validate/transform the DSL state after all basic\nsections/entities have been created. See `Spark.Dsl.Transformer` for more information.\nTransformers are provided as an option to `use`, like so:\n\n    use Spark.Dsl.Extension, sections: [@cars], transformers: [\n      MyApp.Transformers.ValidateNoOverlappingMakesAndModels\n    ]\n\nBy default, the generated modules will have names like `__MODULE__.SectionName.EntityName`, and that could\npotentially conflict with modules you are defining, so you can specify the `module_prefix` option, which would allow\nyou to prefix the modules with something like `__MODULE__.Dsl`, so that the module path generated might be something like\n`__MODULE__.Dsl.SectionName.EntityName`, and you could then have the entity struct be `__MODULE__.SectionName.EntityName`\nwithout conflicts.\n\nTo expose the configuration of your DSL, define functions that use the\nhelpers like `get_entities/2` and `get_opt/3`. For example:\n\n    defmodule MyApp.Cars do\n      def cars(resource) do\n        Spark.Dsl.Extension.get_entities(resource, [:cars])\n      end\n    end\n\n    MyApp.Cars.cars(MyResource)\n    # [%MyApp.Car{...}, %MyApp.Car{...}]\n\nSee the documentation for `Spark.Dsl.Section` and `Spark.Dsl.Entity` for more information","ref":"Spark.Dsl.Extension.html"},{"type":"callback","title":"Spark.Dsl.Extension.add_extensions/0","doc":"","ref":"Spark.Dsl.Extension.html#c:add_extensions/0"},{"type":"function","title":"Spark.Dsl.Extension.doc/2","doc":"","ref":"Spark.Dsl.Extension.html#doc/2"},{"type":"function","title":"Spark.Dsl.Extension.doc_index/2","doc":"","ref":"Spark.Dsl.Extension.html#doc_index/2"},{"type":"function","title":"Spark.Dsl.Extension.expand_alias/2","doc":"","ref":"Spark.Dsl.Extension.html#expand_alias/2"},{"type":"function","title":"Spark.Dsl.Extension.expand_alias_no_require/2","doc":"","ref":"Spark.Dsl.Extension.html#expand_alias_no_require/2"},{"type":"function","title":"Spark.Dsl.Extension.expand_literals/3","doc":"","ref":"Spark.Dsl.Extension.html#expand_literals/3"},{"type":"callback","title":"Spark.Dsl.Extension.explain/1","doc":"","ref":"Spark.Dsl.Extension.html#c:explain/1"},{"type":"function","title":"Spark.Dsl.Extension.fetch_opt/4","doc":"","ref":"Spark.Dsl.Extension.html#fetch_opt/4"},{"type":"function","title":"Spark.Dsl.Extension.get_entities/2","doc":"Get the entities configured for a given section","ref":"Spark.Dsl.Extension.html#get_entities/2"},{"type":"function","title":"Spark.Dsl.Extension.get_entity_dsl_patches/2","doc":"","ref":"Spark.Dsl.Extension.html#get_entity_dsl_patches/2"},{"type":"function","title":"Spark.Dsl.Extension.get_opt/5","doc":"Get an option value for a section at a given path.\n\nChecks to see if it has been overridden via configuration.","ref":"Spark.Dsl.Extension.html#get_opt/5"},{"type":"function","title":"Spark.Dsl.Extension.get_opt_config/3","doc":"","ref":"Spark.Dsl.Extension.html#get_opt_config/3"},{"type":"function","title":"Spark.Dsl.Extension.get_persisted/3","doc":"Get a value that was persisted while transforming or compiling the resource, e.g `:primary_key`","ref":"Spark.Dsl.Extension.html#get_persisted/3"},{"type":"macro","title":"Spark.Dsl.Extension.import_mods/1","doc":"","ref":"Spark.Dsl.Extension.html#import_mods/1"},{"type":"callback","title":"Spark.Dsl.Extension.module_imports/0","doc":"","ref":"Spark.Dsl.Extension.html#c:module_imports/0"},{"type":"function","title":"Spark.Dsl.Extension.monotonic_number/1","doc":"","ref":"Spark.Dsl.Extension.html#monotonic_number/1"},{"type":"function","title":"Spark.Dsl.Extension.run_transformers/4","doc":"","ref":"Spark.Dsl.Extension.html#run_transformers/4"},{"type":"callback","title":"Spark.Dsl.Extension.sections/0","doc":"","ref":"Spark.Dsl.Extension.html#c:sections/0"},{"type":"macro","title":"Spark.Dsl.Extension.set_entity_opt/4","doc":"","ref":"Spark.Dsl.Extension.html#set_entity_opt/4"},{"type":"macro","title":"Spark.Dsl.Extension.set_section_opt/6","doc":"","ref":"Spark.Dsl.Extension.html#set_section_opt/6"},{"type":"function","title":"Spark.Dsl.Extension.shuffle_opts_to_end/3","doc":"","ref":"Spark.Dsl.Extension.html#shuffle_opts_to_end/3"},{"type":"function","title":"Spark.Dsl.Extension.spark_function_info/1","doc":"","ref":"Spark.Dsl.Extension.html#spark_function_info/1"},{"type":"callback","title":"Spark.Dsl.Extension.transformers/0","doc":"","ref":"Spark.Dsl.Extension.html#c:transformers/0"},{"type":"macro","title":"Spark.Dsl.Extension.unimport_mods/1","doc":"","ref":"Spark.Dsl.Extension.html#unimport_mods/1"},{"type":"callback","title":"Spark.Dsl.Extension.verifiers/0","doc":"","ref":"Spark.Dsl.Extension.html#c:verifiers/0"},{"type":"type","title":"Spark.Dsl.Extension.t/0","doc":"","ref":"Spark.Dsl.Extension.html#t:t/0"},{"type":"module","title":"Spark.Dsl.Fragment","doc":"Allows splitting up a DSL into multiple modules, potentially organizing large DSLs\n\nUse the `of` option to expression what your fragment is a fragment of. You can add\nextensions as you would normally to that resource, and they will be added to the\nparent resource.\n\n    defmodule MyApp.Resource.Graphql do\n      use Spark.Dsl.Fragment, of: Ash.Resource, extensions: AshGraphql.Resource\n  \n      graphql do\n        ...\n      end\n    end","ref":"Spark.Dsl.Fragment.html"},{"type":"module","title":"Spark.Dsl.Patch.AddEntity","doc":"Supply this when defining an extension to add entity builders to another extension's section.\n\nFor example\n\n```elixir\n@entity %Spark.Dsl.Entity{\n  ...\n}\n\n@dsl_patch %Spark.Dsl.Patch.AddEntity{section_path: [:foo, :bar], entity: @entity}\n\nuse Spark.Dsl.Extension, dsl_patches: [@dsl_patch]\n```","ref":"Spark.Dsl.Patch.AddEntity.html"},{"type":"type","title":"Spark.Dsl.Patch.AddEntity.t/0","doc":"","ref":"Spark.Dsl.Patch.AddEntity.html#t:t/0"},{"type":"module","title":"Spark.Dsl.Section","doc":"Declares a DSL section.\n\nA dsl section allows you to organize related configurations. All extensions\nconfigure sections, they cannot add DSL builders to the top level. This\nkeeps things organized, and concerns separated.\n\nA section may have nested sections, which will be configured the same as other sections.\nGetting the options/entities of a section is done by providing a path, so you would\nuse the nested path to retrieve that configuration. See `Spark.Dsl.Extension.get_entities/2`\nand `Spark.Dsl.Extension.get_opt/4`.\n\nA section may have entities, which are constructors that produce instances of structs.\nFor more on entities, see `Spark.Dsl.Entity`.\n\nA section may also have a `schema`, which is a `NimbleOptions` schema. Spark will produce\nbuilders for those options, so that they may be configured. They are retrieved with\n`Spark.Dsl.Extension.get_opt/4`.\n\nTo create a section that is available at the top level (i.e not  nested inside of its own name), use\n`top_level?: true`. Remember, however, that this has no effect on sections nested inside of other sections.\n\nFor a full example, see `Spark.Dsl.Extension`.","ref":"Spark.Dsl.Section.html"},{"type":"type","title":"Spark.Dsl.Section.auto_set_fields/0","doc":"","ref":"Spark.Dsl.Section.html#t:auto_set_fields/0"},{"type":"type","title":"Spark.Dsl.Section.describe/0","doc":"User provided documentation.\n\nDocumentation provided in a `Section`'s `describe` field will be included by `Spark` in any generated documentation that includes the `Section`.","ref":"Spark.Dsl.Section.html#t:describe/0"},{"type":"type","title":"Spark.Dsl.Section.docs/0","doc":"Internal field. Not set by user.","ref":"Spark.Dsl.Section.html#t:docs/0"},{"type":"type","title":"Spark.Dsl.Section.entities/0","doc":"","ref":"Spark.Dsl.Section.html#t:entities/0"},{"type":"type","title":"Spark.Dsl.Section.examples/0","doc":"","ref":"Spark.Dsl.Section.html#t:examples/0"},{"type":"type","title":"Spark.Dsl.Section.imports/0","doc":"","ref":"Spark.Dsl.Section.html#t:imports/0"},{"type":"type","title":"Spark.Dsl.Section.links/0","doc":"","ref":"Spark.Dsl.Section.html#t:links/0"},{"type":"type","title":"Spark.Dsl.Section.modules/0","doc":"","ref":"Spark.Dsl.Section.html#t:modules/0"},{"type":"type","title":"Spark.Dsl.Section.name/0","doc":"","ref":"Spark.Dsl.Section.html#t:name/0"},{"type":"type","title":"Spark.Dsl.Section.no_depend_modules/0","doc":"","ref":"Spark.Dsl.Section.html#t:no_depend_modules/0"},{"type":"type","title":"Spark.Dsl.Section.patchable?/0","doc":"","ref":"Spark.Dsl.Section.html#t:patchable?/0"},{"type":"type","title":"Spark.Dsl.Section.sections/0","doc":"","ref":"Spark.Dsl.Section.html#t:sections/0"},{"type":"type","title":"Spark.Dsl.Section.snippet/0","doc":"","ref":"Spark.Dsl.Section.html#t:snippet/0"},{"type":"type","title":"Spark.Dsl.Section.t/0","doc":"","ref":"Spark.Dsl.Section.html#t:t/0"},{"type":"type","title":"Spark.Dsl.Section.top_level?/0","doc":"Determines whether a section can be declared directly in a module.\n\nWhen `top_level?: true`, that Section's DSL can be declared outside of a `do` block in a module.","ref":"Spark.Dsl.Section.html#t:top_level?/0"},{"type":"type","title":"Example - Spark.Dsl.Section.top_level?/0","doc":"A `Section` declared with `top_level?: true`:\n\n```elixir\n@my_section %Spark.Dsl.Section{\n  top_level?: true,\n  name: :my_section,\n  schema: [my_field: [type: :atom, required: true]]\n}\n```\n\nCan be declared like this:\n\n```elixir\ndefmodule MyDslModule do\n  my_field :value\nend\n```\n\nWith `top_level?: false`, the DSL section would need to be declared explicitly/:\n\n```elixir\ndefmodule MyDslModule do\n  my_section do\n    my_field :value\n  end\nend\n```","ref":"Spark.Dsl.Section.html#t:top_level?/0-example"},{"type":"behaviour","title":"Spark.Dsl.Transformer","doc":"A transformer manipulates and/or validates the entire DSL state of a resource.\n\nIt's `transform/1` takes a `map`, which is just the values/configurations at each point\nof the DSL. Don't manipulate it directly, if possible, instead use functions like\n`get_entities/3` and `replace_entity/5` to manipulate it.\n\nUse the `after?/1` and `before?/1` callbacks to ensure that your transformer\nruns either before or after some other transformer.\n\nReturn `true` in `after_compile/0` to have the transformer run in an `after_compile` hook,\nbut keep in mind that no modifications to the dsl structure will be retained, so there is no\nreal point in modifying the dsl that you return.","ref":"Spark.Dsl.Transformer.html"},{"type":"function","title":"Spark.Dsl.Transformer.add_entity/4","doc":"","ref":"Spark.Dsl.Transformer.html#add_entity/4"},{"type":"callback","title":"Spark.Dsl.Transformer.after?/1","doc":"","ref":"Spark.Dsl.Transformer.html#c:after?/1"},{"type":"callback","title":"Spark.Dsl.Transformer.after_compile?/0","doc":"","ref":"Spark.Dsl.Transformer.html#c:after_compile?/0"},{"type":"function","title":"Spark.Dsl.Transformer.async_compile/2","doc":"Runs the function in an async compiler.\n\nUse this for compiling new modules and having them compiled\nefficiently asynchronously.","ref":"Spark.Dsl.Transformer.html#async_compile/2"},{"type":"callback","title":"Spark.Dsl.Transformer.before?/1","doc":"","ref":"Spark.Dsl.Transformer.html#c:before?/1"},{"type":"function","title":"Spark.Dsl.Transformer.build_entity/4","doc":"","ref":"Spark.Dsl.Transformer.html#build_entity/4"},{"type":"function","title":"Spark.Dsl.Transformer.build_entity!/4","doc":"","ref":"Spark.Dsl.Transformer.html#build_entity!/4"},{"type":"function","title":"Spark.Dsl.Transformer.eval/3","doc":"Add a quoted expression to be evaluated in the DSL module's context.\n\nUse this *extremely sparingly*. It should almost never be necessary, unless building certain\nextensions that *require* the module in question to define a given function.\n\nWhat you likely want is either one of the DSL introspection functions, like `Spark.Dsl.Extension.get_entities/2`\nor `Spark.Dsl.Extension.get_opt/5)`. If you simply want to store a custom value that can be retrieved easily, or\ncache some precomputed information onto the resource, use `persist/3`.\n\nProvide the dsl state, bindings that should be unquote-able, and the quoted block\nto evaluate in the module. For example, if we wanted to support a `resource.primary_key()` function\nthat would return the primary key (this is unnecessary, just an example), we might do this:\n\n```elixir\nfields = the_primary_key_fields\n\ndsl_state =\n  Transformer.eval(\n    dsl_state,\n    [fields: fields],\n    quote do\n      def primary_key() do\n        unquote(fields)\n      end\n    end\n  )\n```","ref":"Spark.Dsl.Transformer.html#eval/3"},{"type":"function","title":"Spark.Dsl.Transformer.fetch_option/3","doc":"","ref":"Spark.Dsl.Transformer.html#fetch_option/3"},{"type":"function","title":"Spark.Dsl.Transformer.get_entities/2","doc":"","ref":"Spark.Dsl.Transformer.html#get_entities/2"},{"type":"function","title":"Spark.Dsl.Transformer.get_option/4","doc":"","ref":"Spark.Dsl.Transformer.html#get_option/4"},{"type":"function","title":"Spark.Dsl.Transformer.get_persisted/3","doc":"","ref":"Spark.Dsl.Transformer.html#get_persisted/3"},{"type":"function","title":"Spark.Dsl.Transformer.persist/3","doc":"Saves a value into the dsl config with the given key.\n\nThis can be used to precompute some information and cache it onto the resource,\nor simply store a computed value. It can later be retrieved with `Spark.Dsl.Extension.get_persisted/3`.","ref":"Spark.Dsl.Transformer.html#persist/3"},{"type":"function","title":"Spark.Dsl.Transformer.remove_entity/3","doc":"","ref":"Spark.Dsl.Transformer.html#remove_entity/3"},{"type":"function","title":"Spark.Dsl.Transformer.replace_entity/4","doc":"","ref":"Spark.Dsl.Transformer.html#replace_entity/4"},{"type":"function","title":"Spark.Dsl.Transformer.set_option/4","doc":"","ref":"Spark.Dsl.Transformer.html#set_option/4"},{"type":"function","title":"Spark.Dsl.Transformer.sort/1","doc":"","ref":"Spark.Dsl.Transformer.html#sort/1"},{"type":"callback","title":"Spark.Dsl.Transformer.transform/1","doc":"","ref":"Spark.Dsl.Transformer.html#c:transform/1"},{"type":"behaviour","title":"Spark.Dsl.Verifier","doc":"A verifier gets the dsl state and can return `:ok` or `:error`.\n\nIn a verifier, you can reference and depend on other modules without causing compile time dependencies.","ref":"Spark.Dsl.Verifier.html"},{"type":"function","title":"Spark.Dsl.Verifier.fetch_option/3","doc":"","ref":"Spark.Dsl.Verifier.html#fetch_option/3"},{"type":"function","title":"Spark.Dsl.Verifier.get_entities/2","doc":"","ref":"Spark.Dsl.Verifier.html#get_entities/2"},{"type":"function","title":"Spark.Dsl.Verifier.get_option/3","doc":"","ref":"Spark.Dsl.Verifier.html#get_option/3"},{"type":"function","title":"Spark.Dsl.Verifier.get_option/4","doc":"","ref":"Spark.Dsl.Verifier.html#get_option/4"},{"type":"function","title":"Spark.Dsl.Verifier.get_persisted/2","doc":"","ref":"Spark.Dsl.Verifier.html#get_persisted/2"},{"type":"function","title":"Spark.Dsl.Verifier.get_persisted/3","doc":"","ref":"Spark.Dsl.Verifier.html#get_persisted/3"},{"type":"callback","title":"Spark.Dsl.Verifier.verify/1","doc":"","ref":"Spark.Dsl.Verifier.html#c:verify/1"},{"type":"module","title":"Spark.Dsl.Verifiers.VerifyEntityUniqueness","doc":"Verifies that each entity that has an identifier is unique at each path.","ref":"Spark.Dsl.Verifiers.VerifyEntityUniqueness.html"},{"type":"function","title":"Spark.Dsl.Verifiers.VerifyEntityUniqueness.verify/1","doc":"","ref":"Spark.Dsl.Verifiers.VerifyEntityUniqueness.html#verify/1"},{"type":"module","title":"Spark.OptionsHelpers","doc":"Helpers for working with our superset of nimble options.","ref":"Spark.OptionsHelpers.html"},{"type":"module","title":"Additional Types - Spark.OptionsHelpers","doc":"Spark provides the following additional option types for use when building DSLs or validating options.\n\n- `{:one_of, values}` - maps to `{:in, values}` for backwards compatibility\n- `{:tagged_tuple, tag, inner_type}` - maps to `{tag, type}`\n- `{:spark_behaviour, behaviour}` - expects a module that implements the given behaviour, and can be specified with options, i.e `mod` or `{mod, [opt: :val]}`\n- `{:spark_behaviour, behaviour, builtin_module}` - Same as the above, but also accepts a `builtin_module`. The builtin_module is used to provide additional options for the elixir_sense plugin.\n- `{:spark_function_behaviour, behaviour, {function_mod, arity}}` - expects a module that implements the given behaviour, and can be specified with options, i.e `mod` or `{mod, [opt: :val]}`, that also has a special module that supports being provided an anonymous function or MFA as the `:fun` option.\n- `{:spark_function_behaviour, behaviour, builtin_module, {function_mod, arity}}` - Same as the above, but also accepts a `builtin_module`. The builtin_module is used to provide additional options for the elixir_sense plugin.\n- `{:behaviour, behaviour}` - expects a module that implements a given behaviour.\n- `{:spark, dsl_module}` - expects a module that is a `Spark.Dsl`\n- `{:mfa_or_fun, arity}` - expects a function or MFA of a corresponding arity.\n- `{:spark_type, module, builtin_function}` - a behaviour that defines `builtin_function/0` that returns a list of atoms that map to built in variations of that thing.\n- `{:spark_type, module, builtin_function, templates}` - same as the above, but includes additional templates for elixir_sense autocomplete\n- `:fun` - a function of any arity\n- `:literal` -> any literal value. Maps to `:any`, but is used for documentation.\n- `{:literal, value}` -> exactly the value specified.\n- `:quoted` -> retains the quoted value of the code provided to the option\n- `:quoted_with_env` -> retains the qutoed value of the code provided to the option and the env, in a tuple `{quoted, env}`.\n- `{:wrap_list, type}` -> Allows a single value or a list of values.","ref":"Spark.OptionsHelpers.html#module-additional-types"},{"type":"function","title":"Spark.OptionsHelpers.append_doc!/3","doc":"","ref":"Spark.OptionsHelpers.html#append_doc!/3"},{"type":"function","title":"Spark.OptionsHelpers.docs/1","doc":"Creates markdown documentation for a given schema.","ref":"Spark.OptionsHelpers.html#docs/1"},{"type":"function","title":"Spark.OptionsHelpers.fun/1","doc":"","ref":"Spark.OptionsHelpers.html#fun/1"},{"type":"function","title":"Spark.OptionsHelpers.list_of_atoms/1","doc":"","ref":"Spark.OptionsHelpers.html#list_of_atoms/1"},{"type":"function","title":"Spark.OptionsHelpers.literal_value/2","doc":"","ref":"Spark.OptionsHelpers.html#literal_value/2"},{"type":"function","title":"Spark.OptionsHelpers.make_optional!/2","doc":"","ref":"Spark.OptionsHelpers.html#make_optional!/2"},{"type":"function","title":"Spark.OptionsHelpers.make_required!/2","doc":"","ref":"Spark.OptionsHelpers.html#make_required!/2"},{"type":"function","title":"Spark.OptionsHelpers.map/1","doc":"","ref":"Spark.OptionsHelpers.html#map/1"},{"type":"function","title":"Spark.OptionsHelpers.merge_schemas/3","doc":"Merges two schemas, and sets the `subsection` option on all options on the right\nside.","ref":"Spark.OptionsHelpers.html#merge_schemas/3"},{"type":"function","title":"Spark.OptionsHelpers.mfa_or_fun/2","doc":"","ref":"Spark.OptionsHelpers.html#mfa_or_fun/2"},{"type":"function","title":"Spark.OptionsHelpers.module_and_opts/1","doc":"","ref":"Spark.OptionsHelpers.html#module_and_opts/1"},{"type":"function","title":"Spark.OptionsHelpers.sanitize_schema/1","doc":"","ref":"Spark.OptionsHelpers.html#sanitize_schema/1"},{"type":"function","title":"Spark.OptionsHelpers.set_default!/3","doc":"","ref":"Spark.OptionsHelpers.html#set_default!/3"},{"type":"function","title":"Spark.OptionsHelpers.set_type!/3","doc":"","ref":"Spark.OptionsHelpers.html#set_type!/3"},{"type":"function","title":"Spark.OptionsHelpers.spark_behaviour/2","doc":"","ref":"Spark.OptionsHelpers.html#spark_behaviour/2"},{"type":"function","title":"Spark.OptionsHelpers.spark_function_behaviour/3","doc":"","ref":"Spark.OptionsHelpers.html#spark_function_behaviour/3"},{"type":"function","title":"Spark.OptionsHelpers.struct/2","doc":"","ref":"Spark.OptionsHelpers.html#struct/2"},{"type":"function","title":"Spark.OptionsHelpers.tagged_tuple/4","doc":"","ref":"Spark.OptionsHelpers.html#tagged_tuple/4"},{"type":"function","title":"Spark.OptionsHelpers.validate/2","doc":"Sanitizes the option schema and validates with NimbleOptions.","ref":"Spark.OptionsHelpers.html#validate/2"},{"type":"function","title":"Spark.OptionsHelpers.validate!/2","doc":"Sanitizes the option schema and validates with NimbleOptions.","ref":"Spark.OptionsHelpers.html#validate!/2"},{"type":"function","title":"Spark.OptionsHelpers.wrap_list/3","doc":"","ref":"Spark.OptionsHelpers.html#wrap_list/3"},{"type":"type","title":"Spark.OptionsHelpers.nimble_schema/0","doc":"The schema type, as defined by the [NimbleOptions docs](https://hexdocs.pm/nimble_options/NimbleOptions.html#module-schema-options).","ref":"Spark.OptionsHelpers.html#t:nimble_schema/0"},{"type":"type","title":"Spark.OptionsHelpers.nimble_types/0","doc":"The types as specified by the [NimbleOptions docs](https://hexdocs.pm/nimble_options/NimbleOptions.html#module-types).","ref":"Spark.OptionsHelpers.html#t:nimble_types/0"},{"type":"type","title":"Spark.OptionsHelpers.schema/0","doc":"The actual schema type, as used in sections and entities.","ref":"Spark.OptionsHelpers.html#t:schema/0"},{"type":"type","title":"Spark.OptionsHelpers.type/0","doc":"Spark provides additional types on top of those provided by [NimbleOptions](https://hex.pm/packages/nimble_options).","ref":"Spark.OptionsHelpers.html#t:type/0"},{"type":"exception","title":"Spark.Error.DslError","doc":"Used when a DSL is incorrectly configured.","ref":"Spark.Error.DslError.html"},{"type":"type","title":"Spark.Error.DslError.t/0","doc":"","ref":"Spark.Error.DslError.html#t:t/0"},{"type":"task","title":"mix spark.cheat_sheets","doc":"Creates a set of cheat sheets for each Extension provided\n\nYou will want to include this task with the `--check` flag in your CI pipeline as well.","ref":"Mix.Tasks.Spark.CheatSheets.html"},{"type":"function","title":"Mix.Tasks.Spark.CheatSheets.run/1","doc":"","ref":"Mix.Tasks.Spark.CheatSheets.html#run/1"},{"type":"task","title":"mix spark.cheat_sheets_in_search","doc":"Includes generated cheat sheets in the search bar","ref":"Mix.Tasks.Spark.CheatSheetsInSearch.html"},{"type":"function","title":"Mix.Tasks.Spark.CheatSheetsInSearch.run/1","doc":"","ref":"Mix.Tasks.Spark.CheatSheetsInSearch.html#run/1"},{"type":"task","title":"mix spark.formatter","doc":"Manages a variable called `spark_locals_without_parens` in the .formatter.exs from a list of DSL extensions.","ref":"Mix.Tasks.Spark.Formatter.html"},{"type":"function","title":"Mix.Tasks.Spark.Formatter.all_entity_builders_everywhere/3","doc":"","ref":"Mix.Tasks.Spark.Formatter.html#all_entity_builders_everywhere/3"},{"type":"function","title":"Mix.Tasks.Spark.Formatter.run/1","doc":"","ref":"Mix.Tasks.Spark.Formatter.html#run/1"},{"type":"task","title":"mix spark.replace_doc_links","doc":"Replaces any documentation links with text appropriate for hex docs.\n\nThis makes projects support","ref":"Mix.Tasks.Spark.ReplaceDocLinks.html"},{"type":"function","title":"Mix.Tasks.Spark.ReplaceDocLinks.run/1","doc":"","ref":"Mix.Tasks.Spark.ReplaceDocLinks.html#run/1"},{"type":"module","title":"Spark","doc":"Documentation for `Spark`.","ref":"Spark.html"},{"type":"function","title":"Spark.extensions/1","doc":"Returns the extensions a given DSL uses","ref":"Spark.html#extensions/1"},{"type":"function","title":"Spark.implements_behaviour?/2","doc":"Returns true if the module implements the specified behavior","ref":"Spark.html#implements_behaviour?/2"},{"type":"function","title":"Spark.otp_app/1","doc":"Returns the configured otp_app of a given DSL instance","ref":"Spark.html#otp_app/1"},{"type":"function","title":"Spark.sparks/2","doc":"Returns all modules that implement the specified behaviour for a given otp_app.\n\nShould only be called at runtime, not at compile time, as it will have\ninconsistent results at compile time.","ref":"Spark.html#sparks/2"},{"type":"module","title":"Spark.CheatSheet","doc":"Tools to generate cheat sheets for spark DSLs","ref":"Spark.CheatSheet.html"},{"type":"function","title":"Spark.CheatSheet.cheat_sheet/1","doc":"Generate a cheat sheet for a given DSL","ref":"Spark.CheatSheet.html#cheat_sheet/1"},{"type":"function","title":"Spark.CheatSheet.doc/2","doc":"Generate a markdown bullet list documentation for a list of sections","ref":"Spark.CheatSheet.html#doc/2"},{"type":"function","title":"Spark.CheatSheet.doc_index/3","doc":"Generate a table of contents for a list of sections","ref":"Spark.CheatSheet.html#doc_index/3"},{"type":"function","title":"Spark.CheatSheet.section_cheat_sheet/2","doc":"","ref":"Spark.CheatSheet.html#section_cheat_sheet/2"},{"type":"module","title":"Spark.CodeHelpers","doc":"Helpers for meta programming around code and code snippets","ref":"Spark.CodeHelpers.html"},{"type":"function","title":"Spark.CodeHelpers.code_identifier/1","doc":"Given a section of Elixir AST, generate a hash of the code to help with\ngenerating unique names.","ref":"Spark.CodeHelpers.html#code_identifier/1"},{"type":"function","title":"Spark.CodeHelpers.lift_functions/3","doc":"Lift anonymous and captured functions.\n\nActs as an AST transformer to allow these kinds of functions to be added in\nthe AST:\n\nIn the case of captured functions, it ensures they are all captured remote\nfunctions (ie calls with both the module and function name present) - this\noften requires the definition of a new public function on the target module.\n\nIn the case of anonymous functions, it converts them into a new public\nfunction on the module and returns a (remote) function capture much like that\nof above.","ref":"Spark.CodeHelpers.html#lift_functions/3"},{"type":"module","title":"Spark.Formatter","doc":"Formats Spark modules.\n\nCurrently, it is very simple, and will only reorder the outermost sections according to some rules.\n\n# Plugin\n\nInclude the plugin into your `.formatter.exs` like so `plugins: [Spark.Formatter]`.\n\nIf no configuration is provided, it will sort all top level DSL sections *alphabetically*.\n\n# Section Order\n\nTo provide a custom section order, add configuration to your app, for example:\n\n```elixir\nconfig :spark, :formatter,\n  remove_parens?: true,\n  \"Ash.Resource\": [\n    section_order: [\n      :resource,\n      :postgres,\n      :attributes,\n      :relationships,\n      :aggregates,\n      :calculations\n    ]\n  ],\n  \"MyApp.Resource\": [\n    # Use this if you use a module that is not the spark DSL itself.\n    # For example, you might have a \"base\" that you use instead that sets some simple defaults.\n\n    # This tells us what the actual thing is so we know what extensions are included automatically.\n    type: Ash.Resource,\n\n    # Tell us what extensions might be added under the hood\n    extensions: [MyApp.ResourceExtension],\n    section_order: [...]\n  ]\n```\n\nAny sections found that aren't in that list will be left in the order that they were in, the sections\nin the list will be sorted \"around\" those sections. E.g the following list: `[:code_interface, :attributes]` can be interpreted as\n\"ensure that code_interface comes before attributes, and don't change the rest\".","ref":"Spark.Formatter.html"},{"type":"function","title":"Spark.Formatter.features/1","doc":"","ref":"Spark.Formatter.html#features/1"},{"type":"function","title":"Spark.Formatter.format/2","doc":"","ref":"Spark.Formatter.html#format/2"},{"type":"function","title":"Spark.Formatter.prewalk/2","doc":"Copy of `Macro.prewalk/2` w/ a branch accumulator","ref":"Spark.Formatter.html#prewalk/2"},{"type":"function","title":"Spark.Formatter.prewalk/4","doc":"Copy of `Macro.prewalk/3` w/ a branch accumulator","ref":"Spark.Formatter.html#prewalk/4"},{"type":"function","title":"Spark.Formatter.traverse/5","doc":"A copy of the corresponding `Macro.traverse` function that has a separate accumulator that only goes *down* each branch, only for `pre`","ref":"Spark.Formatter.html#traverse/5"},{"type":"module","title":"Spark.InfoGenerator","doc":"Used to dynamically generate configuration functions for Spark extensions\nbased on their DSL.","ref":"Spark.InfoGenerator.html"},{"type":"module","title":"Usage - Spark.InfoGenerator","doc":"```elixir\ndefmodule MyConfig do\n  use Spark.InfoGenerator, extension: MyDslExtension, sections: [:my_section]\nend\n```","ref":"Spark.InfoGenerator.html#module-usage"},{"type":"macro","title":"Spark.InfoGenerator.generate_config_functions/2","doc":"Given an extension and a list of DSL sections generate individual config\nfunctions for each option.","ref":"Spark.InfoGenerator.html#generate_config_functions/2"},{"type":"macro","title":"Spark.InfoGenerator.generate_entity_functions/2","doc":"Given an extension and a list of DSL sections, generate an entities function\nwhich returns a list of entities.","ref":"Spark.InfoGenerator.html#generate_entity_functions/2"},{"type":"macro","title":"Spark.InfoGenerator.generate_options_functions/2","doc":"Given an extension and a list of DSL sections, generate an options function\nwhich returns a map of all configured options for a resource (including\ndefaults).","ref":"Spark.InfoGenerator.html#generate_options_functions/2"},{"type":"function","title":"Spark.InfoGenerator.spec_for_type/2","doc":"","ref":"Spark.InfoGenerator.html#spec_for_type/2"},{"type":"type","title":"Spark.InfoGenerator.options/0","doc":"","ref":"Spark.InfoGenerator.html#t:options/0"},{"type":"module","title":"Spark.Types","doc":"Helpers for spark types","ref":"Spark.Types.html"},{"type":"function","title":"Spark.Types.doc_type/1","doc":"","ref":"Spark.Types.html#doc_type/1"},{"type":"extras","title":"Writing Extensions","doc":"# Writing Extensions\n\nWriting extensions generally involves three main components.","ref":"writing-extensions.html"},{"type":"extras","title":"The DSL declaration - Writing Extensions","doc":"The DSL is declared as a series of `Spark.Dsl.Section`, which can contain `Spark.Dsl.Entity` and further `Spark.Dsl.Section` structs. See `Spark.Dsl.Section` and `Spark.Dsl.Entity` for more information.","ref":"writing-extensions.html#the-dsl-declaration"},{"type":"extras","title":"Transformers - Writing Extensions","doc":"Extension writing gets a bit more complicated when you get into the world of transformers, but this is also where a lot of the power is. Each transformer can declare other transformers it must go before or after, and then is given the opportunity to modify the entirety of the DSL it is extending up to that point. This allows extensions to make rich modifications to the structure in question. See `Spark.Dsl.Transformer` for more information","ref":"writing-extensions.html#transformers"},{"type":"extras","title":"Introspection - Writing Extensions","doc":"Use functions in `Spark.Dsl.Extension` to retrieve the stored values from the DSL and expose them in a module. The convention is to place functions for something like `MyApp.MyExtension` in `MyApp.MyExtension.Info`. Using introspection functions like this allows for a richer introspection API (i.e not just getting and retrieving raw values), and it also allows us to add type specs and documentation, which is helpful when working generically. I.e `module_as_variable.table()` can't be known by dialyzer, whereas `Extension.table(module)` can be.","ref":"writing-extensions.html#introspection"},{"type":"extras","title":"Get Started With Spark","doc":"# Spark\n\nSpark helps you build powerful and well documented DSLs that come with useful tooling out of the box. DSLs are declared using simple structs, and every DSL has the ability to be extended by the end user. Spark powers all of the DSLs in Ash Framework.\n\nWhat you get for your DSL when you implement it with Spark:\n\n- Extensibility. Anyone can write extensions for your DSL.\n- Autocomplete and in-line documentation: An elixir_sense plugin that \"Just Works\" for any DSL implemented with Spark.\n- Tools to generate documentation for your DSL automatically.\n- A mix task to add every part of the DSL to the `locals_without_parens` of your library automatically.\n\nThis library has only recently been extracted out from Ash core, so there is still work to be done to document and test it in isolation. See the module documentation for the most up-to-date information.","ref":"get-started-with-spark.html"},{"type":"extras","title":"Dependency - Get Started With Spark","doc":"`{:spark, \"~> 1.1.41\"}`","ref":"get-started-with-spark.html#dependency"}],"content_type":"text/markdown"}